/* 
 * EDOMI beta (Version 1.50) - Intelligente Steuerung und Visualisierung KNX-basierter Elektro-Installationen.
 * @link http://www.edomi.de 
 * @author Dr. Christian Gärtner <dr.christian.gaertner@googlemail.com> 
 * @copyright 2015,2016,2017 Dr. Christian Gärtner 
 * @license Freeware 
 *
 *     This file and all other files are part of EDOMI.
 *
 *     EDOMI is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *     Diese und alle anderen Dateien sind Teil von EDOMI.
 *
 *     EDOMI wird in der Hoffnung, dass es nützlich sein wird, aber
 *     OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *     Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
*/ 
/*
============================================================================
Initialisierung
============================================================================
*/

function firstinit(visuid,newSid,version) {
	//dies wird NUR EINMALIG beim allerersten Start aufgerufen (also nach dem Laden der index.php)
	
	//--- ggf. anpassen ---
	displayPixelRatio=2;			//Faktor für die virtuelle Auflösung eines CANVAS (z.B. für Retina-Displays sollte der Faktor mindestens 2 betragen)
									//benötigt für: Diagramme, Rundinstrumente
	//---------------------

	sid=newSid;
	edomiVersion=version;

	visu_visuid=null;				//aktuelle VisuID (wird erst in app1.php/initVisu gesetzt)
	visu_startPageId=null;			//Startseite
	visu_refresh=3; 				//akutelle Refreshzeit (je nach Modus: normal/ss)
	visu_defaultrefresh=3; 			//normale Refreshzeit
	visu_sspageid=null;				//Bildschirmschoner-Seiten-ID
	visu_ssrefresh=3; 				//Bildschirmschoner Refreshzeit
	visu_sstimeout=null; 			//nach dieser Zeit (Minuten!) wird der Bilderschirmschoner aufgerufen
	visu_clickrefresh=1000; 		//Refreshzeit nach einem Klick (in Millisekunden!!)
	visu_cmdQueueLatency=250;		//0=keine Click-Queue verwenden, >0=Wartezeit (ms) nach einem Klick, bis Queue abgearbeitet wird
	visu_cmdQueue='';				//Queue für Visu-Befehle und KO-Setzen
	visu_cmdQueueTimer=null;		//Timer für Queue
	visu_indiColor='#80e000';		//Farbe für alle Indikatoren (Klicks, ZSU, etc.)
	visu_indiColorText='inherit';	//Text-Farbe für einige Indikatoren (Tastatureingabe, Drehregler, etc.)

	visu_touchMode=0;				//Touch oder Click? (0=Automatisch, 1=Klick, 2=Touch)
	visu_allowScrolling=false;		//iOS: Scrollen erlauben?

	visu_isMobile=(navigator.userAgent.match(/iPhone|iPad|iPod|iOS|Android/i)?true:false);	//true=Mobile-Device

	visu_sound=false;				//Audio-Objekt
	visu_soundEnabled=0;			//0=Sound wird nicht benötigt, 1=wird benötigt ist aber noch nicht initialisiert (ggf. iOS), 2=steht zu Verfügung und ist initialisiert
	visu_textToSpeech=false; 		//SpeechSynthesisUtterance-Objekt
	visu_textToSpeechEnabled=0;		//0=TextToSpeech wird nicht benötigt, 1=wird benötigt ist aber noch nicht initialisiert (iOS), 2=steht zu Verfügung und ist initialisiert

	preload_images_max=0;			//Preload-Bilder (Anzahl, nur temporär für den Preloadvorgang!)
	preload_imagesFile=new Array();	//Preload-Bilder (Dateinamen, nur temporär für den Preloadvorgang!)

	initVisu(visuid);
}

function initVisu(visuid) {
	//wird beim Start und Fehler (=Neustart) aufgerufen

	windowZindex=0; 				//Fenster-Verwaltung
	busyID=0; 						//Busy-Fenster (Instanz-Zähler)

	errorTimer=null; 				//Timer für Fehlerbehandlung (serverNotReady)
	screensaverTimer=null; 			//Timer für Bildschirmschoner
	pageTimer=null; 				//Timer für Page-Refresh
	clickTimer=null; 				//Timer für Spezial-Click

	screensaverActive=false;		//false=SS ist gerade inaktiv (Normalzustand), true=SS ist gerade aktiv
	initScreensaverTimer();			//Bildschirmschoner zurücksetzen

	elementDragId=0;				//für Controltypen mit Drag-Funktion
	
	visu_showWarningErr=0;			//Fehlerhinweis quittieren: Enthält die Anzahl der aktuell quittierten Fehler, -1=deaktiviert

	visu_cmdQueue='';				//Klick-Queue leeren

	ajax("initVisu",0,visuid,0,"","");
}

function prepareVisu() {
	document.getElementById("waitanim").style.borderLeftColor="#20e000";
	document.getElementById("waitanim").style.borderRightColor="#20e000";
	if (preload_imagesFile.length>0) {
		document.getElementById("preload").style.display="block";
		preload_images_max=preload_imagesFile.length;
		preloadImagesLoad();
	} else {
		startVisu();
	}
}

function startVisu() {
	document.getElementById("waitanim").style.borderLeftColor="#e00000";
	document.getElementById("waitanim").style.borderRightColor="#e00000";
	document.getElementById("wait").style.display="none";
	document.getElementById("preload").style.display="none";
	document.getElementById("windowContainer").style.display="block";
	initPage(visu_startPageId,true);
}

function preloadImagesLoad() {
	if (preload_imagesFile.length>0) {
		var id=preload_imagesFile[0].match(/visu\/img\/img-([0-9]*)./);
		if (id && id[1]>0) {
			var obj=new Image();			
			obj.onload=function(){preloadImagesReady();}
			obj.onerror=function(){preloadImagesReady();}
			obj.src=preload_imagesFile.shift();
		} else {
			preload_imagesFile.shift();
			preloadImagesReady();		
		}
	} else {
		startVisu();
	}
}

function preloadImagesReady() {
	var t=parseInt((preload_images_max-preload_imagesFile.length)*100/preload_images_max);
	document.getElementById("preload").style.background="-webkit-linear-gradient(left,#ffffff 0%,#ffffff "+t+"%,#606060 "+t+"%,#606060 100%)";
	preloadImagesLoad();
}

function jsLogin(visuid,visus) {
	//Logindialog (modal)	
	gotoStart();
	var loginWinId="login";
	var n="<div id='loginform' class='appWindowLogin' onkeydown='if (event.keyCode==13) {ajax(\"login\",0,\"\",\"\",\"\",controlGetFormData(\"loginform\"));}' style='width:200px;'>";
		n+="<div style='margin-bottom:10px;'><b>Login</b></div>";
		n+="<select id='"+loginWinId+"-loginform-fd0' class='loginSelect' data-type='1' style='margin-bottom:15px;'>"+visus+"</select>";
		n+="<input type='text' id='"+loginWinId+"-loginform-fd1' data-type='1' value='' maxlength='30' class='loginInput' autofocus style='width:100%; height:25px; padding:5px; border-color:transparent;'></input><br>";
		n+="<input type='password' id='"+loginWinId+"-loginform-fd2' data-type='1' value='' maxlength='30' class='loginInput' style='width:100%; height:25px; padding:5px; border-color:transparent;'></input><br>";
		n+="<br>";
		n+="<button id='"+loginWinId+"-loginform-fd3' class='loginCheckbox' onClick='checkboxToggle(this);' data-type='5' data-list='normal|Neuladen' data-value='"+((visu_isMobile)?"1":"0")+"' style='float:left;'></button>";
		n+="<div class='loginButton' onClick='ajax(\"login\",0,\"\",\"\",\"\",controlGetFormData(\"loginform\"));' style='float:right; border-color:transparent;'>Ok</div>";
	n+="</div>";
	
	//POST-Login (für alterative Login-Methode unter iOS & Co.)
	n+="<form id='loginform-post' action='index.php' method='post'><input type='hidden' id='loginvisuid' name='loginvisuid' value=''></input><input type='hidden' id='loginsid' name='loginsid' value=''></input></form>";

	document.getElementById(loginWinId).innerHTML=n;
	checkboxInit(document.getElementById(loginWinId+"-loginform-fd3"));
	document.getElementById(loginWinId).style.display="block";
}


/*
============================================================================
Sounds
============================================================================
*/

function visuSoundInit() {
	if (visu_soundEnabled==1) {
		if (!visu_sound) {visu_sound=new Audio("../shared/etc/snd-empty.mp3");}	//Dummy
		visu_sound.type="audio/mpeg";
		visu_sound.preload="none";
		visu_sound.load();
		visu_sound.autoplay=false;
		visu_sound.loop=false;
		visu_sound.onplay=function(event){visu_soundEnabled=2;}
		visu_sound.play();
	}
}

function visuSoundPlay(sndId,sndUrl,sndLoop) {
	if (visu_sound && visu_soundEnabled==2) {
		if (sndId>0) {sndUrl="../data/liveproject/visu/etc/snd-"+sndId+".mp3";}
		visu_sound.src=sndUrl; 
		visu_sound.load();
		visu_sound.loop=sndLoop;
		visu_sound.play();
	}
}

function visuSoundStop() {
	if (visu_sound && visu_soundEnabled==2) {
		visu_sound.src="../shared/etc/snd-empty.mp3";
		visu_sound.load();
		visu_sound.pause();
	}
}

function visuTextToSpeechInit() {
	if (visu_textToSpeechEnabled==1 && ("speechSynthesis" in window)) {
		if (!visu_textToSpeech) {visu_textToSpeech=new SpeechSynthesisUtterance(" ");}	//Dummy
		visu_textToSpeech.onstart=function(event){visu_textToSpeechEnabled=2;}
		window.speechSynthesis.speak(visu_textToSpeech);
	}
}	

function visuTextToSpeechPlay(voice,language,rate,pitch,msg) {
	if (visu_textToSpeech && visu_textToSpeechEnabled==2 && msg!="") {
		visu_textToSpeech.text=msg;
		if (voice!="") {visu_textToSpeech.voice=voice;}	//### Voice wird noch nicht wirklich unterstützt...
		if (language!="") {visu_textToSpeech.lang=language;}
		if (rate!="") {visu_textToSpeech.rate=rate;}
		if (pitch!="") {visu_textToSpeech.pitch=pitch;}
		window.speechSynthesis.speak(visu_textToSpeech);
	}
}

function visuTextToSpeechStop() {
	if (visu_textToSpeech && visu_textToSpeechEnabled==2) {
		window.speechSynthesis.cancel();
	}
}


/*
============================================================================
Visu und Visu-Seiten/-Popups
============================================================================
*/

function initPage(pageid,initSS) {
	//withoutSS: false=Bildschirmschoner NICHT initialisieren, true=Bildschirmschoner initialisieren

	//Drag-Operationen ggf. absenden
	if (elementDragId>0) {
		control11_Send();
		control12_Send();
		elementDragId=0;
	}
	//Input-Operationen ggf. abbrechen
	document.activeElement.blur();

	if (initSS) {initScreensaverTimer();} //Bildschirmschoner zurücksetzen (jeder Seitenaufruf ist ja Konsequenz einer Aktion, also Click oder Visualarm)
	ajax("initPage",0,visu_visuid,pageid,"","");
}

function refreshPage() {
	//alle Elemente (und deren Value) senden
	var n="";
	var tmp=document.getElementById("windowContainer").querySelectorAll("[data-gaid]");
	for (var t=0; t<tmp.length; t++) {
		n+=tmp[t].dataset.id+AJAX_SEPARATOR2+tmp[t].dataset.gaid+AJAX_SEPARATOR2+tmp[t].dataset.ts+AJAX_SEPARATOR1;
	}
	clearTimeout(pageTimer);
	ajax("refreshPage",0,visu_visuid,0,n,"");
}


/*
============================================================================
Popups
============================================================================
*/

function focusExistingPopup(pageid) {
	//existiert bereits ein Popup(!) mit dieser pageid, wird das Popup in den Vordergrund geholt
	//return: true=Popup ist bereits geöffnet, false=Popup existiert nicht
	var win=document.getElementById("windowContainer").childNodes;
	for (var t=0;t<win.length;t++) {
		if (win[t].dataset.pagetyp==1 && win[t].dataset.pageid==pageid) {
			focusWindow(win[t].id);
			return true;
		}
	}
	return false;
}

function closePopup(winId) {
	if (winId!="w0") {
		//ggf. AutoClose-Timer löschen
		var d=document.getElementById(winId);
		if (d) {
			if (d.dataset.autoclosetimer) {clearTimeout(d.dataset.autoclosetimer);}
		}
		closeWindow(winId);
	}
}

function closeAllPopups() {
	//zunächst die betroffenen WinIDs in tmp[] kopieren (sonst geht es nicht, weil das win-Array beim closeWindow irgendwie zerstört wird...)
	var tmp=new Array();
	var win=document.getElementById("windowContainer").childNodes;
	for (var t=0;t<win.length;t++) {
		if (win[t].dataset.pagetyp==1) {tmp.push(win[t].id);}
	}
	//alle Popups schliessen
	for (var t=0;t<tmp.length;t++) {
		closePopup(tmp[t]);
	}
}


/*
============================================================================
Bildschirmschoner
============================================================================
*/

function startScreensaver() {
	if (visu_sstimeout>0 && visu_sspageid>0) {
		visu_refresh=visu_ssrefresh;
		screensaverActive=true;
		initPage(visu_sspageid,false);
	}
}

function initScreensaverTimer() {
	//wird bei jeder User-Aktion (Click) von checkClick() aufgerufen, und bei jedem Seitenwechsel (normal und Popup, damit ein Visualarm den Bildschirmschoner deaktiviert)
	if (screensaverActive) { //Bildschirmschoner wurde gerade beendet (durch Click oder Visualarm)
		//normale Refresh-Zeit setzen
		visu_refresh=visu_defaultrefresh;
		clearTimeout(pageTimer);
		pageTimer=window.setTimeout("refreshPage();",visu_refresh*1000);
	}

	screensaverActive=false;
	clearTimeout(screensaverTimer);
	if (visu_sstimeout>0 && visu_sspageid>0) {
		screensaverTimer=window.setTimeout("startScreensaver();",visu_sstimeout*60*1000);
	}
}


/*
============================================================================
Fehlerbehandlung
============================================================================
*/

function showWarnings(wCPU,wRAM,wHDD,wERR,wCLK) {
	var n="";
	if (visu_soundEnabled=="1" || visu_textToSpeechEnabled=="1") {n+="<div on"+downEvent+"='ackSound(this);' class='visuWarning' style='color:#000000; background:#80e000; -webkit-animation:none; pointer-events:auto;'>&#9651;<br>Tonausgabe aktivieren</div>";}

	if (visu_showWarningErr>=0 && parseInt(wERR)>visu_showWarningErr) {n+="<div on"+downEvent+"='ackWarningErr(this,"+parseInt(wERR)+");' class='visuWarning' style='color:#ffffff; background:#ff4000; -webkit-animation:none; pointer-events:auto;'>&#9651;<br>"+wERR+" FEHLER</div>";}
	if (wCPU=="1") {n+="<div class='visuWarning'>&#9651;<br>CPU</div>";}
	if (wRAM=="1") {n+="<div class='visuWarning'>&#9651;<br>RAM</div>";}
	if (wHDD=="1") {n+="<div class='visuWarning'>&#9651;<br>HDD</div>";}
	if (wCLK=="1") {n+="<div class='visuWarning'>&#9651;<br>UHRZEIT</div>";}
	if (n!="") {
		document.getElementById("warn").innerHTML=n;
		document.getElementById("warn").style.display="inline";
	} else {
		document.getElementById("warn").style.display="none";
	}	
}

function ackWarningErr(obj,wERR) {
	obj.style.opacity="0.5";
	visu_showWarningErr=wERR;
}

function ackSound(obj) {
	obj.style.opacity="0.5";

	visuSoundInit();
	visuTextToSpeechInit();
}

function gotoStart() {
	//alles auf Neustart

	//Drag-Operationen ggf. absenden
	if (elementDragId>0) {
		control11_Send();
		control12_Send();
		elementDragId=0;
	}
	//Input-Operationen ggf. abbrechen
	document.activeElement.blur();

	//hier alle(!!!) Timer löschen
	//### auch Kameras/Diagramme usw. ###
	clearTimeout(clickTimer);
	clearTimeout(pageTimer);
	clearTimeout(screensaverTimer);
	clearTimeout(errorTimer);

	//alle Fenster entfernen
	clearObject("windowContainer",0);

	//Busy-Fenster entfernen (und Instanz-Zähler auf 0)
	hideBusyWindow();
	busyID=0;

	//alle Warnungen ausblenden
	document.getElementById("warn").style.display="none";

	//Sounds stoppen
	visuSoundStop();
	visuTextToSpeechStop();

	//wait-Animation einblenden
	document.getElementById("wait").style.display="block";
}

function serverNotReady(visuid) {
	//Server ist nicht bereit (db.sysInfo)
	//d.h. php-Serverscripte starten gerade neu (z.B. beim Compilieren...)
	//=> Visu immer wieder starten, bis es klappt

	gotoStart();

	//kompletter Neustart nach x Sekunden
	errorTimer=window.setTimeout("initVisu("+visuid+");",3*1000);
}


/*
============================================================================
Fenster-Verwaltung
============================================================================
*/

function openWindow(xsize,ysize) {
	var winID=newWindow();

	if (winID=="w0") { //w0 (Visu-Seite)
		var n="<div id='"+winID+"-page' class='appWindowNormal' style='width:"+xsize+"px; height:"+ysize+"px; box-shadow:none; -webkit-animation:none;'></div>";
		document.getElementById(winID).style.background="none";
	} else { //w1..x (Popups)
		var n="<table width='100%' height='100%' cellpadding='0' cellspacing='0' border='0'><tr valign='middle'><td>";
			n+="<div id='"+winID+"-page' class='appWindowNormal' style='width:"+xsize+"px; height:"+ysize+"px; border-radius:3px;'></div>";
			n+="</td></tr></table>";
	}
	document.getElementById(winID).innerHTML=n;
	return winID;
}

function focusWindow(winID) {
	//Popup on top bringen
	if (winID!=null) {
		var win=document.getElementById(winID);

		//neuen winID-zindex = ehemals größer zindex
		var maxZ=0;
		var wins=document.getElementById("windowContainer").childNodes;
		for (var t=0;t<wins.length;t++) {
			if (wins[t].style.zIndex>maxZ) {maxZ=parseInt(wins[t].style.zIndex);}
		}

		//alle zindex, die größer sind als win-zindex: dec()
		for (var t=0;t<wins.length;t++) {
			if (wins[t].style.zIndex>win.style.zIndex) {wins[t].style.zIndex=parseInt(wins[t].style.zIndex)-1;}
		}

		win.style.zIndex=maxZ;
	}
}

function closeWindow(winID) {
	if (winID!=null) {

		var win=document.getElementById(winID);

		//alle zindex, die größer sind als win-zindex: dec()
		var wins=document.getElementById("windowContainer").childNodes;
		for (var t=0;t<wins.length;t++) {
			if (wins[t].style.zIndex>win.style.zIndex) {wins[t].style.zIndex=parseInt(wins[t].style.zIndex)-1;}
		}
		windowZindex--;

		if (win) {win.parentNode.removeChild(win);}
		if (document.getElementById("shortclick")) {document.getElementById("shortclick").style.display="none";}
	}
}

function newWindow() {
	if (document.getElementById("windowContainer").lastChild) {
		var newID=parseInt(document.getElementById("windowContainer").lastChild.id.replace("w",""))+1;
	} else {
		var newID=0;
	}
	var winID="w"+newID;
	var div=document.createElement("div");
	document.getElementById("windowContainer").appendChild(div);
	div.className="appWindow";
	div.id=winID;
	windowZindex++;
	div.style.zIndex=windowZindex;
	return winID;
}


function openBusyWindow() {
	document.getElementById("busy").style.display="inline";
	busyID++;
}

function hideBusyWindow() {
	document.getElementById("busy").style.display="none";
	busyID--;
}



/*
============================================================================
Hilfsfunktionen
============================================================================
*/

function getClientDatetime(mode) {
	//mode: 1=Datum, 2=Uhrzeit, 3=Beides mit " / " getrennt, 4=Beides mit " " getrennt
	var d=new Date(); 
	var d1=("0"+d.getDate()).slice(-2)+"."+("0"+(d.getMonth()+1)).slice(-2)+"."+("0000"+d.getFullYear()).slice(-4);
	var d2=("0"+d.getHours()).slice(-2)+":"+("0"+d.getMinutes()).slice(-2)+":"+("0"+d.getSeconds()).slice(-2);
	if (mode==1) {return d1;}
	if (mode==2) {return d2;}
	if (mode==3) {return d1+" / "+d2;}
	if (mode==4) {return d1+" "+d2;}
}

function checkClick() {
	//prüft, ob (z.B. auf iOS-Geräten) mehrere Finger ein touch-Event auslösen
	//(wird bei jeder User-Aktion (Click) aufgerufen und setzt daher auch gleich den Bildschirmschoner-Timer zurück)
	//return: true=Singletouch, false=Multitouch
	initScreensaverTimer();
	if (!event) {var event=window.event;}
	if (event && event.touches) {
		if (event.touches.length>1) {
			return false;
		}
	}
	return true;
}

function getPageX(e) {
	//absolute Mouseposition
	if (e.touches) {
		return e.touches[0].pageX;	//e.changedTouches[0].pageX;
	} else {
		return e.pageX;
	}
}

function getPageY(e) {
	//absolute Mouseposition
	if (e.touches) {
		return e.touches[0].pageY;	//e.changedTouches[0].pageY;
	} else {
		return e.pageY;
	}
}

function getClientX(e) {
	//relative Mouseposition (Viewport)
	if (e.touches) {
		return e.touches[0].clientX;	//e.changedTouches[0].clientX;
	} else {
		return e.clientX;
	}
}

function getClientY(e) {
	//relative Mouseposition (Viewport)
	if (e.touches) {
		return e.touches[0].clientY;	//e.changedTouches[0].clientY;
	} else {
		return e.clientY;
	}
}


function disableScrolling() {
	if (!event) var event=window.event;
	event.preventDefault(); //verhindert das Scrollen auf iOS
}

function getDivPosition(obj) {
	//Liefert die absolute Positions eines Elements zurück (klettert quasi den DOM-Baum nach oben)
	//return: Objekt (z.B. Position.x und Position.y)
	var xPos=0;
	var yPos=0;
	while(obj) {
		xPos+=obj.offsetLeft;
		yPos+=obj.offsetTop;
		obj=obj.offsetParent;
	}
	return [xPos,yPos]; //{x:xPos,y:yPos};
}

function createNewDiv(parentId,id) {
	//erzeugt neues Div und hängt es an parent
	//parentId: Parent-Objekt-ID
	//id: gewünschte ID des neuen DIVs
	var div=document.createElement('div');
	document.getElementById(parentId).appendChild(div);
	div.id=id;
	return div;
}

function clickCancel() {
	//verhindert bei verschachtelten DIVs, dass das Click-Event vom Parent bearbeitet wird, wenn auf das Child geklickt wird
	if (!event) var event=window.event;
	if (event) {
		if (event.cancelBubble) {event.cancelBubble=true;}
		if (event.stopPropagation) {event.stopPropagation();}
	}
}

function clearObject(objId,mode) {
	//Leert ein Objekt (entspricht quasi obj.innerHTML="")
	//mode: 1=setzt obj.style.display auf "none"
	var obj=document.getElementById(objId);
	if (mode==1) {obj.style.display="none";}
	while (obj.firstChild) {obj.removeChild(obj.firstChild);}
}

function shakeObj(objId) {
	var obj=document.getElementById(objId);
	obj.style.webkitAnimation="none";
	obj.style.webkitAnimationPlayState="paused";
	setTimeout(function(){document.getElementById(objId).style.webkitAnimation="animShake 0.3s linear"; document.getElementById(objId).style.webkitAnimationPlayState="running";},100);
}


/*
============================================================================
Ajax
============================================================================
*/

function ajax(cmd,appID,visuID,pageID,data,phpdata) {
	openBusyWindow();
	var req=new XMLHttpRequest();
	req.onreadystatechange=function() {
		if (this.readyState==4 && this.status==200) {
			try {
				eval(this.responseText);
			} catch(e) {
				console.log("Ajax-Response fehlerhaft! Error-Msg: "+e.message+" / URL: "+decodeURIComponent(url)+" / Response: "+this.responseText);
//				if (appID==0) { //nur Neustart, wenn grundsätzliche Funktionalität gefährdet ist (also NICHT bei Kamera, Diagramm, etc.)
//					serverNotReady(visuID);
//				}
			}
			hideBusyWindow();
		}
		if (this.readyState==4 && this.status!=200) {
			console.log("Ajax-Abruf gescheitert! http-status: "+this.status+" / URL: "+decodeURIComponent(url));
			if (appID==0) { //nur Neustart, wenn grundsätzliche Funktionalität gefährdet ist (also NICHT bei Kamera, Diagramm, etc.)
				serverNotReady(visuID);
			}
		}
	}
	//POST
//console.log(data+" /// "+phpdata);
	var url="apps/app"+appID+".php?cmd="+encodeURIComponent(cmd)+"&appid="+encodeURIComponent(appID)+"&visuid="+encodeURIComponent(visuID)+"&pageid="+encodeURIComponent(pageID)+"&datetime="+encodeURIComponent(getClientDatetime(4))+"&sid="+encodeURIComponent(sid)+"&vid="+encodeURIComponent(edomiVersion);
	req.open("POST",url,true);
	req.timeout=30000; //nach 30s: Timeout
	req.setRequestHeader("Content-type","application/x-www-form-urlencoded");
	req.send("data="+encodeURIComponent(data)+"&phpdata="+encodeURIComponent(phpdata));
}


/*
============================================================================
Befehle ausführen, bzw. KOs setzen
============================================================================
*/

function execCmd(elementId) {
	//Befehlsliste ausführen (ggf. in Queue speichern)
	if (visu_cmdQueueLatency>0) {
		if (visu_cmdQueueTimer) {clearTimeout(visu_cmdQueueTimer);}
		visu_cmdQueue+="0"+AJAX_SEPARATOR2+elementId+AJAX_SEPARATOR1;
		visu_cmdQueueTimer=window.setTimeout("sendCmdQueue();",visu_cmdQueueLatency);
	} else {
		clearTimeout(pageTimer);
		pageTimer=window.setTimeout("refreshPage();",visu_clickrefresh);
		ajax("execCmd",1000,visu_visuid,0,elementId,"");
	}
}

function setKoValue(koId,value) {
	//setzt ein KO auf den Wert value (ggf. in Queue speichern)
	if (visu_cmdQueueLatency>0) {
		if (visu_cmdQueueTimer) {clearTimeout(visu_cmdQueueTimer);}
		visu_cmdQueue+="1"+AJAX_SEPARATOR2+koId+AJAX_SEPARATOR2+value+AJAX_SEPARATOR1;
		visu_cmdQueueTimer=window.setTimeout("sendCmdQueue();",visu_cmdQueueLatency);
	} else {
		var n=koId+AJAX_SEPARATOR1+value+AJAX_SEPARATOR1;
		clearTimeout(pageTimer);
		pageTimer=window.setTimeout("refreshPage();",visu_clickrefresh);
		ajax("writeKo",1000,visu_visuid,0,n,"");
	}
}

function sendCmdQueue() {
	//Befehls/KO-Queue senden
	if (visu_cmdQueueTimer) {clearTimeout(visu_cmdQueueTimer);}

	clearTimeout(pageTimer);
	pageTimer=window.setTimeout("refreshPage();",visu_clickrefresh);	//nächster Refresh
	ajax("execCmdQueue",1000,visu_visuid,0,visu_cmdQueue,"");
	visu_cmdQueue="";
}



/*
============================================================================
Controltyp-Funktionen
============================================================================
*/

//allgemeine Controls und Funktionen
//----------------------------

function controlGetFormData(formId) {
	//bildet aus allen Controls in dem "Formular" formId einen String ("1;2;3;...;")
	//die Controls müssen alle eine ID in der Art "x-y-fd#" haben, wobei # eine laufende Nummer ist (fd1,fd2,...)
	//die Reihenfolge ist egal, aus fd1=1,fd3=3,fd2=2 wird "1;2;3;" (NICHT "1;3;2;")
	//fehlende Nummern werden im String aufgefüllt: aus fd1=1,fd2=2,fd4=4 wird "1;2;;4;"
	var n=new Array();
	var tmp=document.getElementById(formId).querySelectorAll("[data-type]");
	for (var t=0; t<tmp.length; t++) {
		var type=tmp[t].dataset.type;
		if (tmp[t].id) {var id=tmp[t].id.split("-")[2].replace("fd","");}
		if (type==1)  {n[id]=stringCleanup(tmp[t].value);}				//Input (Text/Hidden)
		if (type==5)  {n[id]=stringCleanup(tmp[t].dataset.value);}		//Checkbox
		if (type==6)  {n[id]=stringCleanup(tmp[t].dataset.value);}		//Selectbox zum Durchklicken (für TSU)
	}
	var r="";
	for (var t=0; t<n.length; t++) {
		if (n[t]==undefined) {n[t]="";}
		r+=n[t]+AJAX_SEPARATOR1;
	}
	return r;
}

function checkboxInit(obj) {
	if (obj.dataset.value!=0) {
		obj.innerHTML="<button class='elementContent' style='width:100%; height:100%; background:"+visu_indiColor+"; color:"+visu_indiColorText+";'>"+obj.dataset.list.split("|")[1]+"</button>";
	} else {
		obj.innerHTML=obj.dataset.list.split("|")[0];
	}
}

function checkboxToggle(obj) {
	if (obj.dataset.value!=0) {
		obj.dataset.value=0;
	} else {
		obj.dataset.value=1;
	}
	checkboxInit(obj);
}

function selectboxInit(obj) {
	obj.innerHTML=obj.dataset.list.split("|")[obj.dataset.value];
}

function selectboxToggle(obj) {
	var arr=obj.dataset.list.split("|");
	var t=parseInt(obj.dataset.value)+1;
	if (t>=arr.length) {t=0;}
	obj.dataset.value=t;
	selectboxInit(obj);
}

function scrollToTop(objId) {
	document.getElementById(objId).scrollTop=0;
}

function scrollToEnd(objId) {
	var obj=document.getElementById(objId);
	obj.scrollTop = obj.scrollHeight;
}

function scrollUp(objId) {
	var d=document.getElementById(objId);
	d.scrollTop-=parseInt(d.clientHeight);
}

function scrollDown(objId) {
	var d=document.getElementById(objId);
	d.scrollTop+=parseInt(d.clientHeight);
}

function createWheel(parentObj,wheelObj,wheelScale) {
	//Erzeugt einen "Drehknopf" und passt die Größe und Position an (Verhältnis immer 1:1 und zentriert)
	//parentObj: Container-Objekt (Rad wird entsprechend dimensioniert)
	//wheelObj: das eigentliche Rad-Objekt
	//wheelScale: Anzahl der Pixel, um die das Rad vergrößert/verkleinert werden soll (gegenüber dem parentObj), normalerweise=0

	wheelWidth=3; //Rahmenbreite

	var w=parseFloat(parentObj.style.width.replace("px",""));
	var h=parseFloat(parentObj.style.height.replace("px",""));
	var wheelSize=w;
	if (w>=h) {var wheelSize=h;}
	wheelSize+=wheelScale;

	
	wheelObj.style.border=wheelWidth+"px solid "+visu_indiColor;
	wheelObj.style.borderRadius="100%";

	wheelObj.style.left=(w/2-wheelSize/2)+"px";
	wheelObj.style.top=(h/2-wheelSize/2)+"px";
	wheelObj.style.width=(wheelSize-(2*wheelWidth))+"px"; //-(2*wheelWidth) wegen Border
	wheelObj.style.height=(wheelSize-(2*wheelWidth))+"px";
}

function showShortClickItem(obj) {
	//blendet einen Klick-Indikator für 200 Millisekunden ein
	if (obj) {
		if (!obj.dataset.shortClickTimer) {
			obj.className="shortClick";
		} else {
			clearTimeout(obj.dataset.shortClickTimer);
		}
		obj.dataset.shortClickTimer=window.setTimeout("showShortClickItemReset('"+obj.id+"');",200);
	}
}

function showShortClickItemReset(elementId) {
	var d=document.getElementById(elementId);
	if (d) {
		d.className="";
		d.dataset.shortClickTimer="";
	}
}

function visuElementClick(elementId,hideClick) {
	var obj=document.getElementById("e-"+elementId);
	if (!hideClick) {showShortClickItem(obj);}

	if (obj.dataset.hascmd>0) { 			//Befehle
		execCmd(obj.dataset.id);
	}
	if (obj.dataset.gotopageid>0) { 		//Seitenwechsel
		initPage(obj.dataset.gotopageid,true);
	}
	if (obj.dataset.closepopup>0) { 		//Popup schliessen
		closePopup(obj.parentNode.dataset.winid);
	}

	clickCancel();
}
